
import io
import sys
from contextlib import contextmanager, asynccontextmanager
from typing import Callable, Optional

class CaptureOutput:
    def __init__(self, callback: Callable[[str], None]):
        self.callback = callback
        self._stdout = sys.stdout
        self._stderr = sys.stderr
        self._stdout_buffer = io.StringIO()
        self._stderr_buffer = io.StringIO()

    def _capture_output(self, prefix, buffer):
        buffer.seek(0)
        for line in buffer:
            self.callback(f"[{prefix}] {line.strip()}")

    def _start_capture(self):
        sys.stdout = self._stdout_buffer
        sys.stderr = self._stderr_buffer

    def _stop_capture(self):
        self._capture_output("stdout", self._stdout_buffer)
        self._capture_output("stderr", self._stderr_buffer)
        sys.stdout = self._stdout
        sys.stderr = self._stderr

    @contextmanager
    def context(self):
        try:
            self._start_capture()
            yield
        finally:
            self._stop_capture()

    @asynccontextmanager
    async def async_context(self):
        try:
            self._start_capture()
            yield
        finally:
            self._stop_capture()

    def __enter__(self):
        self._start_capture()

    def __exit__(self, exc_type, exc_val, exc_tb):
        self._stop_capture()

    async def __aenter__(self):
        self._start_capture()

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        self._stop_capture()
